@@ -319,8 +319,8 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_RSA_NULL_SHA256,
 	      GNUTLS_CIPHER_NULL,
-	      GNUTLS_KX_RSA, GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_KX_RSA, GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 
 	/* RSA */
 	ENTRY(GNUTLS_RSA_ARCFOUR_128_SHA1,
@@ -346,12 +346,12 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 
 	ENTRY(GNUTLS_RSA_CAMELLIA_128_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC, GNUTLS_KX_RSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_RSA_CAMELLIA_256_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_256_CBC, GNUTLS_KX_RSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_RSA_CAMELLIA_128_CBC_SHA1,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC, GNUTLS_KX_RSA,
 	      GNUTLS_MAC_SHA1, GNUTLS_SSL3,
@@ -362,12 +362,12 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_RSA_AES_128_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_RSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_RSA_AES_256_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_RSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 /* GCM */
 	ENTRY(GNUTLS_RSA_AES_128_GCM_SHA256,
 	      GNUTLS_CIPHER_AES_128_GCM, GNUTLS_KX_RSA,
@@ -418,13 +418,13 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	ENTRY(GNUTLS_DHE_DSS_CAMELLIA_128_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC,
 	      GNUTLS_KX_DHE_DSS,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_DHE_DSS_CAMELLIA_256_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_256_CBC,
 	      GNUTLS_KX_DHE_DSS,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 
 	ENTRY(GNUTLS_DHE_DSS_CAMELLIA_128_CBC_SHA1,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC,
@@ -438,12 +438,12 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_DHE_DSS_AES_128_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_DHE_DSS,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_DHE_DSS_AES_256_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_DHE_DSS,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 /* GCM */
 	ENTRY(GNUTLS_DHE_DSS_AES_128_GCM_SHA256,
 	      GNUTLS_CIPHER_AES_128_GCM, GNUTLS_KX_DHE_DSS,
@@ -478,13 +478,13 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	ENTRY(GNUTLS_DHE_RSA_CAMELLIA_128_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC,
 	      GNUTLS_KX_DHE_RSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_DHE_RSA_CAMELLIA_256_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_256_CBC,
 	      GNUTLS_KX_DHE_RSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_DHE_RSA_CAMELLIA_128_CBC_SHA1,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC,
 	      GNUTLS_KX_DHE_RSA,
@@ -497,12 +497,12 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_DHE_RSA_AES_128_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_DHE_RSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_DHE_RSA_AES_256_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_DHE_RSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 /* GCM */
 	ENTRY(GNUTLS_DHE_RSA_AES_128_GCM_SHA256,
 	      GNUTLS_CIPHER_AES_128_GCM, GNUTLS_KX_DHE_RSA,
@@ -541,20 +541,20 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY_PRF(GNUTLS_ECDHE_RSA_AES_256_CBC_SHA384,
 		  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_ECDHE_RSA,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY(GNUTLS_ECDHE_RSA_ARCFOUR_128_SHA1,
 	      GNUTLS_CIPHER_ARCFOUR, GNUTLS_KX_ECDHE_RSA,
 	      GNUTLS_MAC_SHA1, GNUTLS_SSL3,
 	      GNUTLS_VERSION_UNKNOWN),
 	ENTRY(GNUTLS_ECDHE_RSA_CAMELLIA_128_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC, GNUTLS_KX_ECDHE_RSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_ECDHE_RSA_CAMELLIA_256_CBC_SHA384,
 		  GNUTLS_CIPHER_CAMELLIA_256_CBC, GNUTLS_KX_ECDHE_RSA,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 
 	/* ECDHE-ECDSA */
 	ENTRY(GNUTLS_ECDHE_ECDSA_NULL_SHA1,
@@ -579,23 +579,23 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_VERSION_UNKNOWN),
 	ENTRY(GNUTLS_ECDHE_ECDSA_CAMELLIA_128_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC, GNUTLS_KX_ECDHE_ECDSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_ECDHE_ECDSA_CAMELLIA_256_CBC_SHA384,
 		  GNUTLS_CIPHER_CAMELLIA_256_CBC, GNUTLS_KX_ECDHE_ECDSA,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 
 	/* More ECC */
 
 	ENTRY(GNUTLS_ECDHE_ECDSA_AES_128_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_ECDHE_ECDSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_ECDHE_RSA_AES_128_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_ECDHE_RSA,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_ECDHE_ECDSA_CAMELLIA_128_GCM_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_GCM, GNUTLS_KX_ECDHE_ECDSA,
 	      GNUTLS_MAC_AEAD, GNUTLS_TLS1_2,
@@ -622,8 +622,8 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY_PRF(GNUTLS_ECDHE_ECDSA_AES_256_CBC_SHA384,
 		  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_ECDHE_ECDSA,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 
 	ENTRY(GNUTLS_ECDHE_RSA_CAMELLIA_128_GCM_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_GCM, GNUTLS_KX_ECDHE_RSA,
@@ -669,12 +669,12 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_ECDHE_PSK_AES_128_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_ECDHE_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_ECDHE_PSK_AES_256_CBC_SHA384,
 		  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_ECDHE_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY(GNUTLS_ECDHE_PSK_ARCFOUR_128_SHA1,
 	      GNUTLS_CIPHER_ARCFOUR, GNUTLS_KX_ECDHE_PSK,
 	      GNUTLS_MAC_SHA1, GNUTLS_SSL3,
@@ -685,20 +685,20 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_ECDHE_PSK_NULL_SHA256,
 	      GNUTLS_CIPHER_NULL, GNUTLS_KX_ECDHE_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_ECDHE_PSK_NULL_SHA384,
 		  GNUTLS_CIPHER_NULL, GNUTLS_KX_ECDHE_PSK,
 		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY(GNUTLS_ECDHE_PSK_CAMELLIA_128_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC, GNUTLS_KX_ECDHE_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_ECDHE_PSK_CAMELLIA_256_CBC_SHA384,
 		  GNUTLS_CIPHER_CAMELLIA_256_CBC, GNUTLS_KX_ECDHE_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 
 	ENTRY(GNUTLS_ECDHE_PSK_SALSA20_256_SHA1,
 	      GNUTLS_CIPHER_SALSA20_256, GNUTLS_KX_ECDHE_PSK,
@@ -729,8 +729,8 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_PSK_AES_128_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_PSK_AES_256_GCM_SHA384,
 		  GNUTLS_CIPHER_AES_256_GCM, GNUTLS_KX_PSK,
 		  GNUTLS_MAC_AEAD, GNUTLS_TLS1_2,
@@ -755,16 +755,16 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_PSK_NULL_SHA256,
 	      GNUTLS_CIPHER_NULL, GNUTLS_KX_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_PSK_CAMELLIA_128_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC, GNUTLS_KX_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_PSK_CAMELLIA_256_CBC_SHA384,
 		  GNUTLS_CIPHER_CAMELLIA_256_CBC, GNUTLS_KX_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 
 	ENTRY(GNUTLS_PSK_SALSA20_256_SHA1,
 	      GNUTLS_CIPHER_SALSA20_256, GNUTLS_KX_PSK,
@@ -777,12 +777,12 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY_PRF(GNUTLS_PSK_AES_256_CBC_SHA384,
 		  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY_PRF(GNUTLS_PSK_NULL_SHA384,
 		  GNUTLS_CIPHER_NULL, GNUTLS_KX_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 
 	/* RSA-PSK */
 	ENTRY(GNUTLS_RSA_PSK_ARCFOUR_128_SHA1,
@@ -817,36 +817,36 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_RSA_PSK_AES_128_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_RSA_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_RSA_PSK_NULL_SHA1,
 	      GNUTLS_CIPHER_NULL, GNUTLS_KX_RSA_PSK,
 	      GNUTLS_MAC_SHA1, GNUTLS_TLS1,
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_RSA_PSK_NULL_SHA256,
 	      GNUTLS_CIPHER_NULL, GNUTLS_KX_RSA_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_RSA_PSK_AES_256_GCM_SHA384,
 		  GNUTLS_CIPHER_AES_256_GCM, GNUTLS_KX_RSA_PSK,
 		  GNUTLS_MAC_AEAD, GNUTLS_TLS1_2,
 		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY_PRF(GNUTLS_RSA_PSK_AES_256_CBC_SHA384,
 		  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_RSA_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY_PRF(GNUTLS_RSA_PSK_NULL_SHA384,
 		  GNUTLS_CIPHER_NULL, GNUTLS_KX_RSA_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY(GNUTLS_RSA_PSK_CAMELLIA_128_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC, GNUTLS_KX_RSA_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_RSA_PSK_CAMELLIA_256_CBC_SHA384,
 		  GNUTLS_CIPHER_CAMELLIA_256_CBC, GNUTLS_KX_RSA_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 
 
 	/* DHE-PSK */
@@ -868,8 +868,8 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_DHE_PSK_AES_128_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_DHE_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_DHE_PSK_AES_128_GCM_SHA256,
 	      GNUTLS_CIPHER_AES_128_GCM, GNUTLS_KX_DHE_PSK,
 	      GNUTLS_MAC_AEAD, GNUTLS_TLS1_2,
@@ -880,28 +880,28 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_DHE_PSK_NULL_SHA256,
 	      GNUTLS_CIPHER_NULL, GNUTLS_KX_DHE_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_DHE_PSK_NULL_SHA384,
 		  GNUTLS_CIPHER_NULL, GNUTLS_KX_DHE_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY_PRF(GNUTLS_DHE_PSK_AES_256_CBC_SHA384,
 		  GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_DHE_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY_PRF(GNUTLS_DHE_PSK_AES_256_GCM_SHA384,
 		  GNUTLS_CIPHER_AES_256_GCM, GNUTLS_KX_DHE_PSK,
 		  GNUTLS_MAC_AEAD, GNUTLS_TLS1_2,
 		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY(GNUTLS_DHE_PSK_CAMELLIA_128_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC, GNUTLS_KX_DHE_PSK,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY_PRF(GNUTLS_DHE_PSK_CAMELLIA_256_CBC_SHA384,
 		  GNUTLS_CIPHER_CAMELLIA_256_CBC, GNUTLS_KX_DHE_PSK,
-		  GNUTLS_MAC_SHA384, GNUTLS_TLS1,
-		  GNUTLS_DTLS_VERSION_MIN, GNUTLS_MAC_SHA384),
+		  GNUTLS_MAC_SHA384, GNUTLS_TLS1_2,
+		  GNUTLS_DTLS1_2, GNUTLS_MAC_SHA384),
 	ENTRY(GNUTLS_DHE_PSK_CAMELLIA_128_GCM_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_GCM, GNUTLS_KX_DHE_PSK,
 	      GNUTLS_MAC_AEAD, GNUTLS_TLS1_2,
@@ -932,13 +932,13 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	ENTRY(GNUTLS_DH_ANON_CAMELLIA_128_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC,
 	      GNUTLS_KX_ANON_DH,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_DH_ANON_CAMELLIA_256_CBC_SHA256,
 	      GNUTLS_CIPHER_CAMELLIA_256_CBC,
 	      GNUTLS_KX_ANON_DH,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_DH_ANON_CAMELLIA_128_CBC_SHA1,
 	      GNUTLS_CIPHER_CAMELLIA_128_CBC,
 	      GNUTLS_KX_ANON_DH,
@@ -951,12 +951,12 @@ static const gnutls_cipher_suite_entry cs_algorithms[] = {
 	      GNUTLS_DTLS_VERSION_MIN),
 	ENTRY(GNUTLS_DH_ANON_AES_128_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_128_CBC, GNUTLS_KX_ANON_DH,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_DH_ANON_AES_256_CBC_SHA256,
 	      GNUTLS_CIPHER_AES_256_CBC, GNUTLS_KX_ANON_DH,
-	      GNUTLS_MAC_SHA256, GNUTLS_TLS1,
-	      GNUTLS_DTLS_VERSION_MIN),
+	      GNUTLS_MAC_SHA256, GNUTLS_TLS1_2,
+	      GNUTLS_DTLS1_2),
 	ENTRY(GNUTLS_DH_ANON_AES_128_GCM_SHA256,
 	      GNUTLS_CIPHER_AES_128_GCM, GNUTLS_KX_ANON_DH,
 	      GNUTLS_MAC_AEAD, GNUTLS_TLS1_2,
