@@ -35,7 +35,10 @@
 #include <gnutls_datum.h>
 
 
-int _gnutls_proc_psk_client_kx(gnutls_session_t, uint8_t *, size_t);
+static int _gnutls_proc_psk_client_kx(gnutls_session_t, uint8_t *, size_t);
+static int
+_gnutls_proc_psk_server_kx(gnutls_session_t session, uint8_t * data,
+			   size_t _data_size);
 
 
 const mod_auth_st psk_auth_struct = {
@@ -163,7 +166,7 @@ _gnutls_gen_psk_client_kx(gnutls_session_t session,
 	gnutls_psk_client_credentials_t cred;
 
 	cred = (gnutls_psk_client_credentials_t)
-	    _gnutls_get_cred(session, GNUTLS_CRD_PSK, NULL);
+	    _gnutls_get_cred(session, GNUTLS_CRD_PSK);
 
 	if (cred == NULL) {
 		gnutls_assert();
@@ -199,7 +202,7 @@ _gnutls_gen_psk_client_kx(gnutls_session_t session,
 
 /* just read the username from the client key exchange.
  */
-int
+static int
 _gnutls_proc_psk_client_kx(gnutls_session_t session, uint8_t * data,
 			   size_t _data_size)
 {
@@ -210,7 +213,7 @@ _gnutls_proc_psk_client_kx(gnutls_session_t session, uint8_t * data,
 	psk_auth_info_t info;
 
 	cred = (gnutls_psk_server_credentials_t)
-	    _gnutls_get_cred(session, GNUTLS_CRD_PSK, NULL);
+	    _gnutls_get_cred(session, GNUTLS_CRD_PSK);
 
 	if (cred == NULL) {
 		gnutls_assert();
@@ -234,7 +237,11 @@ _gnutls_proc_psk_client_kx(gnutls_session_t session, uint8_t * data,
 
 	/* copy the username to the auth info structures
 	 */
-	info = _gnutls_get_auth_info(session);
+	info = _gnutls_get_auth_info(session, GNUTLS_CRD_PSK);
+	if (info == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
 
 	if (username.size > MAX_USERNAME_SIZE) {
 		gnutls_assert();
@@ -283,7 +290,7 @@ _gnutls_gen_psk_server_kx(gnutls_session_t session,
 	gnutls_datum_t hint;
 
 	cred = (gnutls_psk_server_credentials_t)
-	    _gnutls_get_cred(session, GNUTLS_CRD_PSK, NULL);
+	    _gnutls_get_cred(session, GNUTLS_CRD_PSK);
 
 	if (cred == NULL) {
 		gnutls_assert();
@@ -306,7 +313,7 @@ _gnutls_gen_psk_server_kx(gnutls_session_t session,
 
 /* just read the hint from the server key exchange.
  */
-int
+static int
 _gnutls_proc_psk_server_kx(gnutls_session_t session, uint8_t * data,
 			   size_t _data_size)
 {
@@ -317,7 +324,7 @@ _gnutls_proc_psk_server_kx(gnutls_session_t session, uint8_t * data,
 	psk_auth_info_t info;
 
 	cred = (gnutls_psk_client_credentials_t)
-	    _gnutls_get_cred(session, GNUTLS_CRD_PSK, NULL);
+	    _gnutls_get_cred(session, GNUTLS_CRD_PSK);
 
 	if (cred == NULL) {
 		gnutls_assert();
@@ -340,7 +347,11 @@ _gnutls_proc_psk_server_kx(gnutls_session_t session, uint8_t * data,
 
 	/* copy the hint to the auth info structures
 	 */
-	info = _gnutls_get_auth_info(session);
+	info = _gnutls_get_auth_info(session, GNUTLS_CRD_PSK);
+	if (info == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
 
 	if (hint.size > MAX_USERNAME_SIZE) {
 		gnutls_assert();
@@ -350,15 +361,8 @@ _gnutls_proc_psk_server_kx(gnutls_session_t session, uint8_t * data,
 	memcpy(info->hint, hint.data, hint.size);
 	info->hint[hint.size] = 0;
 
-	ret = _gnutls_set_psk_session_key(session, &cred->key, NULL);
-	if (ret < 0) {
-		gnutls_assert();
-		goto error;
-	}
-
 	ret = 0;
 
-      error:
 	return ret;
 }
 
