@@ -321,7 +321,7 @@ int gnutls_bye(gnutls_session_t session, gnutls_close_request_t how)
 			do {
 				ret =
 				    _gnutls_recv_int(session, GNUTLS_ALERT,
-						     -1, NULL, 0, NULL,
+						     -1, NULL, NULL, 0, NULL,
 						     session->internals.
 						     record_timeout_ms);
 			}
@@ -349,12 +349,6 @@ int gnutls_bye(gnutls_session_t session, gnutls_close_request_t how)
 	return 0;
 }
 
-inline static void session_invalidate(gnutls_session_t session)
-{
-	session->internals.invalid_connection = 1;
-}
-
-
 inline static void session_unresumable(gnutls_session_t session)
 {
 	session->internals.resumable = RESUME_FALSE;
@@ -515,7 +509,7 @@ _gnutls_send_tlen_int(gnutls_session_t session, content_type_t type,
 		/* Adjust header length and add sequence for DTLS */
 		if (IS_DTLS(session))
 			memcpy(&headers[3],
-			       &record_state->sequence_number.i, 8);
+			       record_state->sequence_number.i, 8);
 
 		_gnutls_record_log
 		    ("REC[%p]: Preparing Packet %s(%d) with length: %d and min pad: %d\n",
@@ -635,6 +629,34 @@ check_buffers(gnutls_session_t session, content_type_t type,
 	return 0;
 }
 
+/* Checks and retrieves any pending data in the application data record buffers.
+ */
+static int
+check_packet_buffers(gnutls_session_t session, content_type_t type,
+		     gnutls_packet_t *packet)
+{
+	if (_gnutls_record_buffer_get_size(session) > 0) {
+		int ret;
+		ret =
+		    _gnutls_record_buffer_get_packet(type, session,
+					      	     packet);
+		if (ret < 0) {
+			if (IS_DTLS(session)) {
+				if (ret == GNUTLS_E_UNEXPECTED_PACKET) {
+					ret = GNUTLS_E_AGAIN;
+				}
+			}
+			gnutls_assert();
+			return ret;
+		}
+
+		return ret;
+	}
+
+	*packet = NULL;
+	return 0;
+}
+
 
 
 /* Here we check if the advertized version is the one we
@@ -753,6 +775,12 @@ record_add_to_buffers(gnutls_session_t session,
 			     gnutls_alert_get_name((int) bufel->msg.
 						   data[1]));
 
+			if (!session->internals.initial_negotiation_completed &&
+			    session->internals.handshake_in_progress && STATE == STATE0) { /* handshake hasn't started */
+				ret = gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET);
+				goto unexpected_packet;
+			}
+
 			session->internals.last_alert = bufel->msg.data[1];
 
 			/* if close notify is received and
@@ -769,7 +797,6 @@ record_add_to_buffers(gnutls_session_t session,
 				/* if the alert is FATAL or WARNING
 				 * return the apropriate message
 				 */
-
 				gnutls_assert();
 				ret = GNUTLS_E_WARNING_ALERT_RECEIVED;
 				if (bufel->msg.data[0] == GNUTLS_AL_FATAL) {
@@ -1337,7 +1364,12 @@ _gnutls_recv_in_buffers(gnutls_session_t session, content_type_t type,
 		return 0;
 	}
 
-	if (IS_DTLS(session)) {
+	if (IS_DTLS(session) && (ret == GNUTLS_E_DECRYPTION_FAILED ||
+		ret == GNUTLS_E_UNSUPPORTED_VERSION_PACKET ||
+		ret == GNUTLS_E_UNEXPECTED_PACKET_LENGTH ||
+		ret == GNUTLS_E_UNEXPECTED_PACKET ||
+		ret == GNUTLS_E_ERROR_IN_FINISHED_PACKET ||
+		ret == GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET)) {
 		goto discard;
 	}
 
@@ -1360,12 +1392,13 @@ _gnutls_recv_in_buffers(gnutls_session_t session, content_type_t type,
 ssize_t
 _gnutls_recv_int(gnutls_session_t session, content_type_t type,
 		 gnutls_handshake_description_t htype,
+		 gnutls_packet_t *packet,
 		 uint8_t * data, size_t data_size, void *seq,
 		 unsigned int ms)
 {
 	int ret;
 
-	if ((type != GNUTLS_ALERT && type != GNUTLS_HEARTBEAT)
+	if (packet == NULL && (type != GNUTLS_ALERT && type != GNUTLS_HEARTBEAT)
 	    && (data_size == 0 || data == NULL))
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
@@ -1389,23 +1422,110 @@ _gnutls_recv_int(gnutls_session_t session, content_type_t type,
 	case RECV_STATE_0:
 
 		_dtls_async_timer_check(session);
-		/* If we have enough data in the cache do not bother receiving
-		 * a new packet. (in order to flush the cache)
-		 */
-		ret = check_buffers(session, type, data, data_size, seq);
-		if (ret != 0)
-			return ret;
 
-		ret = _gnutls_recv_in_buffers(session, type, htype, ms);
-		if (ret < 0 && ret != GNUTLS_E_SESSION_EOF)
-			return gnutls_assert_val(ret);
+		if (packet == NULL) {
+			/* If we have enough data in the cache do not bother receiving
+			 * a new packet. (in order to flush the cache)
+			 */
+			ret = check_buffers(session, type, data, data_size, seq);
+			if (ret != 0)
+				return ret;
 
-		return check_buffers(session, type, data, data_size, seq);
+			ret = _gnutls_recv_in_buffers(session, type, htype, ms);
+			if (ret < 0 && ret != GNUTLS_E_SESSION_EOF)
+				return gnutls_assert_val(ret);
+
+			return check_buffers(session, type, data, data_size, seq);
+		} else {
+			ret = check_packet_buffers(session, type, packet);
+			if (ret != 0)
+				return ret;
+
+			ret = _gnutls_recv_in_buffers(session, type, -1, ms);
+			if (ret < 0 && ret != GNUTLS_E_SESSION_EOF)
+				return gnutls_assert_val(ret);
+
+			return check_packet_buffers(session, type, packet);
+		}
 	default:
 		return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 	}
 }
 
+/**
+ * gnutls_packet_get:
+ * @packet: is a #gnutls_packet_t structure.
+ * @data: will contain the data present in the @packet structure (may be %NULL)
+ * @sequence: the 8-bytes of the packet sequence number (may be %NULL)
+ *
+ * This function returns the data and sequence number associated with
+ * the received packet.
+ *
+ * Since: 3.3.5
+ **/
+
+void gnutls_packet_get(gnutls_packet_t packet, gnutls_datum_t *data, unsigned char *sequence)
+{
+	if (unlikely(packet == NULL)) {
+		gnutls_assert();
+		if (data) {
+			data->data = NULL;
+			data->size = 0;
+		}
+	}
+
+	if (sequence) {
+		memcpy(sequence, packet->record_sequence.i, 8);
+	}
+
+	if (data) {
+		data->size = packet->msg.size - packet->mark;
+		data->data = packet->msg.data + packet->mark;
+	}
+}
+
+/**
+ * gnutls_packet_deinit:
+ * @packet: is a pointer to a #gnutls_packet_st structure.
+ *
+ * This function will deinitialize all data associated with
+ * the received packet.
+ *
+ * Since: 3.3.5
+ **/
+void gnutls_packet_deinit(gnutls_packet_t packet)
+{
+	gnutls_free(packet);
+}
+
+/**
+ * gnutls_record_recv_packet:
+ * @session: is a #gnutls_session_t structure.
+ * @packet: the structure that will hold the packet data
+ *
+ * This is a lower-level function thatn gnutls_record_recv() and allows
+ * to directly receive the whole decrypted packet. That avoids a
+ * memory copy, and is mostly applicable to applications seeking high
+ * performance.
+ *
+ * The received packet is accessed using gnutls_packet_get() and 
+ * must be deinitialized using gnutls_packet_deinit(). The returned
+ * packet will be %NULL if the return value is zero (EOF).
+ *
+ * Returns: The number of bytes received and zero on EOF (for stream
+ * connections).  A negative error code is returned in case of an error.  
+ *
+ * Since: 3.3.5
+ **/
+ssize_t
+gnutls_record_recv_packet(gnutls_session_t session, 
+		   	  gnutls_packet_t *packet)
+{
+	return _gnutls_recv_int(session, GNUTLS_APPLICATION_DATA, -1, packet,
+				NULL, 0, NULL,
+				session->internals.record_timeout_ms);
+}
+
 /**
  * gnutls_record_send:
  * @session: is a #gnutls_session_t structure.
@@ -1472,12 +1592,11 @@ gnutls_record_send(gnutls_session_t session, const void *data,
 }
 
 /**
- * gnutls_cork:
+ * gnutls_record_cork:
  * @session: is a #gnutls_session_t structure.
  *
- * If called gnutls_record_send() will no longer send partial records.
- * All queued records will be sent when gnutls_uncork() is called, or
- * when the maximum record size is reached.
+ * If called, gnutls_record_send() will no longer send any records.
+ * Any sent records will be cached until gnutls_record_uncork() is called.
  *
  * This function is safe to use with DTLS after GnuTLS 3.3.0.
  *
@@ -1489,11 +1608,11 @@ void gnutls_record_cork(gnutls_session_t session)
 }
 
 /**
- * gnutls_uncork:
+ * gnutls_record_uncork:
  * @session: is a #gnutls_session_t structure.
  * @flags: Could be zero or %GNUTLS_RECORD_WAIT
  *
- * This resets the effect of gnutls_cork(), and flushes any pending
+ * This resets the effect of gnutls_record_cork(), and flushes any pending
  * data. If the %GNUTLS_RECORD_WAIT flag is specified then this
  * function will block until the data is sent or a fatal error
  * occurs (i.e., the function will retry on %GNUTLS_E_AGAIN and
@@ -1586,8 +1705,8 @@ int gnutls_record_uncork(gnutls_session_t session, unsigned int flags)
 ssize_t
 gnutls_record_recv(gnutls_session_t session, void *data, size_t data_size)
 {
-	return _gnutls_recv_int(session, GNUTLS_APPLICATION_DATA, -1, data,
-				data_size, NULL,
+	return _gnutls_recv_int(session, GNUTLS_APPLICATION_DATA, -1, NULL,
+				data, data_size, NULL,
 				session->internals.record_timeout_ms);
 }
 
@@ -1615,8 +1734,8 @@ ssize_t
 gnutls_record_recv_seq(gnutls_session_t session, void *data,
 		       size_t data_size, unsigned char *seq)
 {
-	return _gnutls_recv_int(session, GNUTLS_APPLICATION_DATA, -1, data,
-				data_size, seq,
+	return _gnutls_recv_int(session, GNUTLS_APPLICATION_DATA, -1, NULL,
+				data, data_size, seq,
 				session->internals.record_timeout_ms);
 }
 
