@@ -24,8 +24,8 @@
 #include <gnutls/abstract.h>
 
 int _gnutls_x509_cert_verify_peers(gnutls_session_t session,
-				   const char *hostname,
-				   const char *purpose,
+				   gnutls_typed_vdata_st * data,
+			           unsigned int elements,
 				   unsigned int *status);
 
 #define PEM_CERT_SEP2 "-----BEGIN X509 CERTIFICATE"
