@@ -27,9 +27,14 @@
 #include <gmp.h>
 #include <nettle/dsa.h>
 #include <nettle/sha2.h>
+#include <fips.h>
 
 #define MAX_PVP_SEED_SIZE 256
 
+#ifdef USE_NETTLE3
+# define dsa_public_key dsa_params
+#endif
+
 #define div_ceil(x,y) ((x+(y)-1)/(y))
 
 struct dss_params_validation_seeds {
@@ -62,7 +67,12 @@ dsa_generate_dss_pqg(struct dsa_public_key *pub,
 
 int
 dsa_generate_dss_keypair(struct dsa_public_key *pub,
+#ifdef USE_NETTLE3
+		     mpz_t y,
+		     mpz_t x,
+#else
 		     struct dsa_private_key *key,
+#endif
 		     void *random_ctx, nettle_random_func *random,
 		     void *progress_ctx, nettle_progress_func *progress);
 
@@ -79,7 +89,7 @@ int
 _dsa_validate_dss_g(struct dsa_public_key *pub,
 		    unsigned domain_seed_size, const uint8_t *domain_seed, unsigned index);
 
-unsigned _dsa_check_qp_sizes(unsigned q_bits, unsigned p_bits);
+unsigned _dsa_check_qp_sizes(unsigned q_bits, unsigned p_bits, unsigned generate);
 
 /* The following low-level functions can be used for DH key exchange as well 
  */
@@ -98,7 +108,12 @@ _dsa_generate_dss_g(struct dsa_public_key *pub,
 
 void
 _dsa_generate_dss_xy(struct dsa_public_key *pub,
+#ifdef USE_NETTLE3
+		     mpz_t y,
+		     mpz_t x,
+#else
 		     struct dsa_private_key *key,
+#endif
 		     void *random_ctx, nettle_random_func *random);
 
 #define DIGEST_SIZE SHA384_DIGEST_SIZE
@@ -114,4 +129,6 @@ hash (uint8_t digest[DIGEST_SIZE], unsigned length, void *data)
   return;
 }
 
+unsigned mpz_seed_sizeinbase_256_u(mpz_t s, unsigned nominal);
+
 #endif /* DSA_FIPS_H_INCLUDED */
