@@ -31,7 +31,7 @@
 
 #include "utils.h"
 
-#ifdef ENABLE_FIPS140
+#if defined(ENABLE_FIPS140) || defined(ENABLE_ARCFOUR128)
 void doit(void)
 {
 	exit(77);
@@ -100,21 +100,21 @@ try_prio(const char *prio, unsigned expected_cs, unsigned expected_ciphers)
 
 void doit(void)
 {
-	const int normal = 66;
+	const int normal = 61;
 	const int null = 5;
 	const int sec128 = 56;
 
-	try_prio("NORMAL", normal, 10);
-	try_prio("NORMAL:-MAC-ALL:+MD5:+MAC-ALL", normal, 10);
-	try_prio("NORMAL:+CIPHER-ALL", normal, 10);	/* all (except null) */
+	try_prio("NORMAL", normal, 9);
+	try_prio("NORMAL:-MAC-ALL:+MD5:+MAC-ALL", normal, 9);
+	try_prio("NORMAL:+CIPHER-ALL", normal, 9);	/* all (except null) */
 	try_prio("NORMAL:-CIPHER-ALL:+NULL", null, 1);	/* null */
-	try_prio("NORMAL:-CIPHER-ALL:+NULL:+CIPHER-ALL", normal + null, 11);	/* should be null + all */
+	try_prio("NORMAL:-CIPHER-ALL:+NULL:+CIPHER-ALL", normal + null, 10);	/* should be null + all */
 	try_prio("NORMAL:-CIPHER-ALL:+NULL:+CIPHER-ALL:-CIPHER-ALL:+AES-128-CBC", 10, 1);	/* should be null + all */
-	try_prio("PERFORMANCE", normal, 10);
+	try_prio("PERFORMANCE", normal, 9);
 	try_prio("SECURE256", 20, 4);
 	try_prio("SECURE128", sec128, 8);
 	try_prio("SECURE128:+SECURE256", sec128, 8);	/* should be the same as SECURE128 */
-	try_prio("SECURE128:+SECURE256:+NORMAL", normal, 10);	/* should be the same as NORMAL */
+	try_prio("SECURE128:+SECURE256:+NORMAL", normal, 9);	/* should be the same as NORMAL */
 	try_prio("SUITEB192", 1, 1);
 }
 
